// This file is part of parrocchetto.

// parrocchetto is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// parrocchetto is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with parrocchetto.  If not, see <http://www.gnu.org/licenses/>.

#include "flash.h"
#include "spi1.h"
#include <xc.h>
#include <stdint.h>

inline void flash_AT45DB041D_init()
{
    spi1_init();
}

uint8_t flash_AT45DB041D_identify(void)
{
uint8_t buf[4];

    spi1_begin();
    spi1_write(0x9F); // Device identification
    spi1_readbuf(buf, 4);
    spi1_end();
            
    return (buf[0] == 0x1F && buf[1] == 0x24 && buf[2] == 0 && buf[3] == 0);
}

uint8_t flash_AT45DB041D_getstatus(void)
{
uint8_t status;

    spi1_begin();
    spi1_write(0xD7); // Device identification
    status = spi1_read();
    spi1_end();
    return status;
}

void flash_AT45DB041D_erase()
{
uint8_t buf[] = { 0xC7, 0x94, 0x80, 0x9A };

    spi1_begin();
    spi1_writebuf(buf, 4); // Erase all
    spi1_end();
}

void flash_AT45DB041D_pageread(uint16_t addr, uint8_t* page)
{
uint8_t buf[] = { 0xD2, addr >> 8, addr, 0, 0, 0, 0, 0 };

    //while(!(flash_AT45DB041D_getstatus() & 0x80));
    
    spi1_begin();
    spi1_writebuf(buf, 8); // Read page + 24 bit address
    spi1_readbuf(page, 256);
    spi1_end();
}

void flash_AT45DB041D_pagewrite(uint16_t addr, const uint8_t* page)
{
uint8_t buf[] = { 0x82, addr >> 8, addr, 0 };

    spi1_begin();
    spi1_writebuf(buf, 4); // Load buffer 1 + 24 bit address
    spi1_writebuf(page, 256);
    spi1_end();
}

inline void flash_W25Q32JV_enable_write(void)
{
    spi1_begin();
    spi1_write(0x06); // Write enable
    spi1_end();
}


inline void flash_W25Q32JV_init()
{
    spi1_init();
    PORTBbits.RB4 = 1; // ~RST/~HOLD
}

uint8_t flash_W25Q32JV_identify(void)
{
uint8_t buf[3];

    spi1_begin();
    spi1_write(0x9F); // Device identification
    spi1_readbuf(buf, 3);
    spi1_end();
            
    return (buf[0] == 0xEF && buf[1] == 0x40 && buf[2] == 0x16);
}

uint8_t flash_W25Q32JV_getstatus1(void)
{
uint8_t status;

    spi1_begin();
    spi1_write(0x05); // Read status register 1
    status = spi1_read();
    spi1_end();
            
    return status;
}

inline uint8_t flash_W25Q32JV_isbusy(void)
{
    return (flash_W25Q32JV_getstatus1() & 0x1);
}

void flash_W25Q32JV_busywait(void)
{
uint8_t status;

    spi1_begin();
    spi1_write(0x05); // Read status register 1
    do {
        status = spi1_read();
    } while(status & 0x1);
    spi1_end();
}

void flash_W25Q32JV_read(uint16_t page_addr, uint8_t byte_addr, uint8_t* buf, uint8_t size)
{
uint8_t cmd[] = { 0x03, page_addr >> 8, page_addr, byte_addr };
    
    spi1_begin();
    spi1_writebuf(cmd, 4); // Read page + 24 bit address
    spi1_readbuf(buf, size);
    spi1_end();
}

void flash_W25Q32JV_pageread(uint16_t addr, uint8_t* page)
{
uint8_t buf[] = { 0x03, addr >> 8, addr, 0 };
    
    spi1_begin();
    spi1_writebuf(buf, 4); // Read page + 24 bit address
    spi1_readbuf(page, 256);
    spi1_end();
}

void flash_W25Q32JV_pagewrite(uint16_t addr, const uint8_t* page)
{
uint8_t buf[] = { 0x02, addr >> 8, addr, 0 };

    flash_W25Q32JV_enable_write();
    
    spi1_begin();
    spi1_writebuf(buf, 4); // Load buffer 1 + 24 bit address
    spi1_writebuf(page, 256);
    spi1_end();
}

void flash_W25Q32JV_sectorerase(uint16_t addr)
{
uint8_t buf[] = { 0x20, addr >> 8, addr, 0 };

    flash_W25Q32JV_enable_write();

    spi1_begin();
    spi1_writebuf(buf, 4); // Sector erase
    spi1_end();
}

void flash_W25Q32JV_erase()
{
   flash_W25Q32JV_enable_write();

    spi1_begin();
    spi1_write(0xC7); // Chip erase
    spi1_end();

    flash_W25Q32JV_busywait();
}